const Tree = require('gittypes/tree')
const Blob = require('gittypes/blob')

module.exports = function (blob) {
  return blob.bind(Tree, function (buffer) {
    return new Tree(function (api) {
      return getZipFile(buffer).then(function (zipfile) {
        return getEntries(zipfile).then(function (entries) {
          var tree = entries.reduce(function (tree, entry) {
            if (/\/$/.test(entry.fileName)) return tree
            tree[entry.fileName] = entryToBlob(zipfile, entry)
            return tree
          }, {})
          return mkTree(tree).getHash(api)
        })
      })
    })
  })
}

function getZipFile (buffer) {
  return new Promise(function (resolve, reject) {
    var yauzl = require('yauzl')
    yauzl.fromBuffer(buffer, {lazyEntries: true}, function (err, zipfile) {
      if (err) return reject(err)
      resolve(zipfile)
    })
  })
}
function getEntries (zipfile) {
  return new Promise(function (resolve, reject) {
    var entries = []
    zipfile.on('error', (err) => reject(err))
    zipfile.on('entry', function (entry) {
      entries.push(entry)
      zipfile.readEntry()
    })
    zipfile.on('end', () => resolve(entries))
    zipfile.readEntry()
  })
}
function entryToBlob (zipfile, entry) {
  return new Blob(function (api) {
    return new Promise(function (resolve, reject) {
      zipfile.openReadStream(entry, function (err, readStream) {
        if (err) return reject(err)
        var buffs = []
        readStream.on('data', function (buff) {
          buffs.push(buff)
        })
        readStream.on('error', function (err) {
          reject(err)
        })
        readStream.on('end', function () {
          resolve(Buffer.concat(buffs))
        })
        readStream.resume()
      })
    }).then((buffer) => Blob.of(buffer).getHash(api))
  })
}
function mkTree (treeDict) {
  var next = {}
  var tree = Object.keys(treeDict).reduce(function (tree, absPath) {
    var [name, ...rest] = absPath.split('/')
    if (rest.length === 0) {
      tree[name] = treeDict[name]
    } else {
      next[name] = next[name] || {}
      next[name][rest.join('/')] = treeDict[absPath]
    }
    return tree
  }, {})
  tree = Object.keys(next).reduce(function (tree, name) {
    tree[name] = mkTree(next[name])
    return tree
  }, tree)
  return Tree.of(tree)
}
